@@ -1,12 +1,13 @@
-from __future__ import unicode_literals, print_function
+from __future__ import unicode_literals
 
-import datetime, operator, sys
+import datetime, operator
 from optparse import make_option
 
 from django.core.management.base import BaseCommand
 from django.core.management.base import CommandError
 from django.db.models import Q, Count
 from django.contrib.contenttypes.models import ContentType
+from django.utils.six.moves import input
 
 from reversion.models import Revision, Version
 from django.db.utils import DatabaseError
@@ -16,12 +17,12 @@ class Command(BaseCommand):
     option_list = BaseCommand.option_list + (
         make_option("--date", "-t",
             dest="date",
-            help="Delete only revisions older then the specify date. The date should be a valid date given in the ISO format (YYYY-MM-DD)"),
+            help="Delete only revisions older than the specify date. The date should be a valid date given in the ISO format (YYYY-MM-DD)"),
         make_option("--days", "-d",
             dest="days",
             default=0,
             type="int",
-            help="Delete only revisions older then the specify number of days."),
+            help="Delete only revisions older than the specify number of days."),
         make_option("--keep-revision", "-k",
             dest="keep",
             default=0,
@@ -61,11 +62,11 @@ Examples:
     
         deleterevisions myapp.mymodel --days=365 --force
         
-    That will delete every revision of myapp.model that are older then 365 days, even if there's revisions involving other apps and/or models.
+    That will delete every revision of myapp.model that are older than 365 days, even if there's revisions involving other apps and/or models.
     
         deleterevisions myapp.mymodel --keep=10
         
-    That will delete only revisions of myapp.model if there's more then 10 revisions for an object, keeping the 10 most recent revisons.
+    That will delete only revisions of myapp.model if there's more than 10 revisions for an object, keeping the 10 most recent revisons.
 """
 
     def handle(self, *app_labels, **options):
@@ -148,7 +149,7 @@ Examples:
                 else:
                     objs = objs.exclude(content_type__in=models)
 
-            # Get all the objects that have more then the maximum revisions
+            # Get all the objects that have more than the maximum revisions
             objs = objs.values("object_id", "content_type_id").annotate(total_ver=Count("object_id")).filter(total_ver__gt=keep)
 
             revisions_not_keeped = set()
@@ -195,11 +196,7 @@ Examples:
 
         # Ask confirmation
         if confirmation:
-            try:
-                raw_input = raw_input
-            except NameError:
-                raw_input = input
-            choice = raw_input("Are you sure you want to delete theses revisions? [y|N] ")
+            choice = input("Are you sure you want to delete theses revisions? [y|N] ")
             if choice.lower() != "y":
                 print("Aborting revision deletion.")
                 return
