@@ -2,23 +2,32 @@
 
 from __future__ import unicode_literals
 
-import operator, sys
+import operator, warnings
 from functools import wraps, reduce, partial
 from threading import local
 from weakref import WeakValueDictionary
 import copy
+from collections import defaultdict
+
+try:
+    from django.apps import apps
+    get_model = apps.get_model
+except ImportError:  # For Django < 1.7  pragma: no cover
+    from django.db.models import get_model
 
 from django.contrib.contenttypes.models import ContentType
 from django.core import serializers
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.signals import request_finished
 from django.db import models, connection, transaction
-from django.db.models import Q, Max, get_model
+from django.db.models import Q, Max
 from django.db.models.query import QuerySet
 from django.db.models.signals import post_save
 from django.utils.encoding import force_text
 
-from reversion.models import Revision, Version, has_int_pk, pre_revision_commit, post_revision_commit
+from reversion.models import Revision, Version, has_int_pk
+from reversion.signals import pre_revision_commit, post_revision_commit
+from reversion.errors import RevisionManagementError, RegistrationError
 
 
 class VersionAdapter(object):
@@ -67,14 +76,14 @@ class VersionAdapter(object):
             # Get the referenced obj(s).
             try:
                 related = getattr(obj, relationship)
-            except ObjectDoesNotExist:
+            except ObjectDoesNotExist:  # pragma: no cover
                 continue
             if isinstance(related, models.Model):
                 yield related
             elif isinstance(related, (models.Manager, QuerySet)):
                 for related_obj in related.all():
                     yield related_obj
-            elif related is not None:
+            elif related is not None:  # pragma: no cover
                 raise TypeError("Cannot follow the relationship {relationship}. Expected a model or QuerySet, found {related}".format(
                     relationship = relationship,
                     related = related,
@@ -110,9 +119,23 @@ class VersionAdapter(object):
         }
 
 
-class RevisionManagementError(Exception):
+class RevisionContextStackFrame(object):
 
-    """Exception that is thrown when something goes wrong with revision managment."""
+    def __init__(self, is_managing_manually, is_invalid=False, ignore_duplicates=False):
+        self.is_managing_manually = is_managing_manually
+        self.is_invalid = is_invalid
+        self.ignore_duplicates = ignore_duplicates
+        self.objects = defaultdict(dict)
+        self.meta = []
+
+    def fork(self, is_managing_manually):
+        return RevisionContextStackFrame(is_managing_manually, self.is_invalid, self.ignore_duplicates)
+
+    def join(self, other_context):
+        if not other_context.is_invalid:
+            for manager_name, object_versions in other_context.objects.items():
+                self.objects[manager_name].update(object_versions)
+            self.meta.extend(other_context.meta)
 
 
 class RevisionContextManager(local):
@@ -127,29 +150,25 @@ class RevisionContextManager(local):
 
     def clear(self):
         """Puts the revision manager back into its default state."""
-        self._objects = {}
         self._user = None
         self._comment = ""
         self._stack = []
-        self._is_invalid = False
-        self._meta = []
-        self._ignore_duplicates = False
         self._db = None
 
     def is_active(self):
         """Returns whether there is an active revision for this thread."""
         return bool(self._stack)
 
-    def is_managing_manually(self):
-        """Returns whether this revision context has manual management enabled."""
-        self._assert_active()
-        return self._stack[-1]
-
     def _assert_active(self):
         """Checks for an active revision, throwning an exception if none."""
-        if not self.is_active():
+        if not self.is_active():  # pragma: no cover
             raise RevisionManagementError("There is no active revision for this thread")
 
+    @property
+    def _current_frame(self):
+        self._assert_active()
+        return self._stack[-1]
+
     def start(self, manage_manually=False):
         """
         Begins a revision for this thread.
@@ -158,50 +177,39 @@ class RevisionContextManager(local):
         leave these methods alone and instead use the revision context manager
         or the `create_revision` decorator.
         """
-        self._stack.append(manage_manually)
+        if self.is_active():
+            self._stack.append(self._current_frame.fork(manage_manually))
+        else:
+            self._stack.append(RevisionContextStackFrame(manage_manually))
 
     def end(self):
         """Ends a revision for this thread."""
         self._assert_active()
-        self._stack.pop()
-        if not self._stack:
+        stack_frame = self._stack.pop()
+        if self._stack:
+            self._current_frame.join(stack_frame)
+        else:
             try:
-                if not self.is_invalid():
+                if not stack_frame.is_invalid:
                     # Save the revision data.
-                    for manager, manager_context in self._objects.items():
+                    for manager, manager_context in stack_frame.objects.items():
                         manager.save_revision(
                             dict(
                                 (obj, callable(data) and data() or data)
                                 for obj, data
                                 in manager_context.items()
+                                if obj.pk is not None
                             ),
                             user = self._user,
                             comment = self._comment,
-                            meta = self._meta,
-                            ignore_duplicates = self._ignore_duplicates,
+                            meta = stack_frame.meta,
+                            ignore_duplicates = stack_frame.ignore_duplicates,
                             db = self._db,
                         )
             finally:
                 self.clear()
 
-    def invalidate(self):
-        """Marks this revision as broken, so should not be commited."""
-        self._assert_active()
-        self._is_invalid = True
-
-    def is_invalid(self):
-        """Checks whether this revision is invalid."""
-        return self._is_invalid
-
-    def add_to_context(self, manager, obj, version_data):
-        """Adds an object to the current revision."""
-        self._assert_active()
-        try:
-            manager_context = self._objects[manager]
-        except KeyError:
-            manager_context = {}
-            self._objects[manager] = manager_context
-        manager_context[obj] = version_data
+    # Revision context properties that apply to the entire stack.
 
     def get_db(self):
         """Returns the current DB alias being used."""
@@ -231,20 +239,35 @@ class RevisionContextManager(local):
         self._assert_active()
         return self._comment
 
+    # Revision context properties that apply to the current stack frame.
+
+    def is_managing_manually(self):
+        """Returns whether this revision context has manual management enabled."""
+        return self._current_frame.is_managing_manually
+
+    def invalidate(self):
+        """Marks this revision as broken, so should not be commited."""
+        self._current_frame.is_invalid = True
+
+    def is_invalid(self):
+        """Checks whether this revision is invalid."""
+        return self._current_frame.is_invalid
+
+    def add_to_context(self, manager, obj, version_data):
+        """Adds an object to the current revision."""
+        self._current_frame.objects[manager][obj] = version_data
+
     def add_meta(self, cls, **kwargs):
         """Adds a class of meta information to the current revision."""
-        self._assert_active()
-        self._meta.append((cls, kwargs))
+        self._current_frame.meta.append((cls, kwargs))
 
     def set_ignore_duplicates(self, ignore_duplicates):
         """Sets whether to ignore duplicate revisions."""
-        self._assert_active()
-        self._ignore_duplicates = ignore_duplicates
+        self._current_frame.ignore_duplicates = ignore_duplicates
 
     def get_ignore_duplicates(self):
         """Gets whether to ignore duplicate revisions."""
-        self._assert_active()
-        return self._ignore_duplicates
+        return self._current_frame.ignore_duplicates
 
     # Signal receivers.
 
@@ -254,7 +277,7 @@ class RevisionContextManager(local):
         are closed. Not closing all active revisions can cause memory leaks
         and weird behaviour.
         """
-        while self.is_active():
+        while self.is_active():  # pragma: no cover
             self.end()
 
     # High-level context management.
@@ -302,11 +325,6 @@ class RevisionContext(object):
 revision_context_manager = RevisionContextManager()
 
 
-class RegistrationError(Exception):
-
-    """Exception thrown when registration with django-reversion goes wrong."""
-
-
 class RevisionManager(object):
 
     """Manages the configuration and creation of revisions."""
@@ -323,12 +341,12 @@ class RevisionManager(object):
         """Returns the manager with the given slug."""
         if manager_slug in cls._created_managers:
             return cls._created_managers[manager_slug]
-        raise RegistrationError("No revision manager exists with the slug %r" % manager_slug)
+        raise RegistrationError("No revision manager exists with the slug %r" % manager_slug)  # pragma: no cover
 
     def __init__(self, manager_slug, revision_context_manager=revision_context_manager):
         """Initializes the revision manager."""
         # Check the slug is unique for this revision manager.
-        if manager_slug in RevisionManager._created_managers:
+        if manager_slug in RevisionManager._created_managers:  # pragma: no cover
             raise RegistrationError("A revision manager has already been created with the slug %r" % manager_slug)
         # Store a reference to this manager.
         self.__class__._created_managers[manager_slug] = self
@@ -540,14 +558,10 @@ class RevisionManager(object):
 
         The results are returned with the most recent versions first.
         """
-        versions = self.get_for_object(obj, db)
-        changed_versions = []
-        last_serialized_data = None
-        for version in versions:
-            if last_serialized_data != version.serialized_data:
-                changed_versions.append(version)
-            last_serialized_data = version.serialized_data
-        return changed_versions
+        warnings.warn(
+            "Use get_for_object().get_unique() instead of get_unique_for_object().",
+            PendingDeprecationWarning)
+        return list(self.get_for_object(obj, db).get_unique())
 
     def get_for_date(self, object, date, db=None):
         """Returns the latest version of an object for the given date."""
@@ -590,7 +604,7 @@ class RevisionManager(object):
         ).values_list("latest_pk", flat=True)
         # HACK: MySQL deals extremely badly with this as a subquery, and can hang infinitely.
         # TODO: If a version is identified where this bug no longer applies, we can add a version specifier.
-        if connection.vendor == "mysql":
+        if connection.vendor == "mysql":  # pragma: no cover
             deleted_version_pks = list(deleted_version_pks)
         # Return the deleted versions!
         return self._get_versions(db).filter(pk__in=deleted_version_pks).order_by("-pk")
@@ -619,3 +633,35 @@ class RevisionManager(object):
 
 # A shared revision manager.
 default_revision_manager = RevisionManager("default")
+
+
+# Easy registration methods.
+register = default_revision_manager.register
+is_registered = default_revision_manager.is_registered
+unregister = default_revision_manager.unregister
+get_adapter = default_revision_manager.get_adapter
+get_registered_models = default_revision_manager.get_registered_models
+
+
+# Context management.
+create_revision = revision_context_manager.create_revision
+
+
+# Revision meta data.
+get_db = revision_context_manager.get_db
+set_db = revision_context_manager.set_db
+get_user = revision_context_manager.get_user
+set_user = revision_context_manager.set_user
+get_comment = revision_context_manager.get_comment
+set_comment = revision_context_manager.set_comment
+add_meta = revision_context_manager.add_meta
+get_ignore_duplicates = revision_context_manager.get_ignore_duplicates
+set_ignore_duplicates = revision_context_manager.set_ignore_duplicates
+
+
+# Low level API.
+get_for_object_reference = default_revision_manager.get_for_object_reference
+get_for_object = default_revision_manager.get_for_object
+get_unique_for_object = default_revision_manager.get_unique_for_object
+get_for_date = default_revision_manager.get_for_date
+get_deleted = default_revision_manager.get_deleted
