@@ -1,5 +1,10 @@
+from django.contrib.contenttypes.models import ContentType
 from django.db import models
 from django.utils.encoding import force_text, python_2_unicode_compatible
+try:
+    from django.contrib.contenttypes.fields import GenericRelation, GenericForeignKey
+except ImportError:  # Django < 1.9  pragma: no cover
+    from django.contrib.contenttypes.generic import GenericRelation, GenericForeignKey
 
 from reversion.models import Revision
 
@@ -31,6 +36,11 @@ def get_str_pk():
     return force_text(str_pk_gen)
 
 
+class ReversionTestModel1Child(ReversionTestModel1):
+
+    pass
+
+
 class ReversionTestModel2(ReversionTestModelBase):
 
     id = models.CharField(
@@ -72,12 +82,16 @@ class RevisionMeta(models.Model):
 
 # Admin test models.
 
+@python_2_unicode_compatible
 class ParentTestAdminModel(models.Model):
 
     parent_name = models.CharField(
         max_length = 200,
     )
 
+    def __str__(self):
+        return self.parent_name
+
 
 @python_2_unicode_compatible
 class ChildTestAdminModel(ParentTestAdminModel):
@@ -90,7 +104,18 @@ class ChildTestAdminModel(ParentTestAdminModel):
         return self.child_name
 
 
-class InlineTestParentModel(models.Model):
+@python_2_unicode_compatible
+class InlineTestChildGenericModel(models.Model):
+
+    object_id = models.IntegerField(
+        db_index = True,
+    )
+
+    content_type = models.ForeignKey(
+        ContentType,
+    )
+
+    object = GenericForeignKey()
 
     name = models.CharField(
         max_length = 100,
@@ -100,6 +125,20 @@ class InlineTestParentModel(models.Model):
         return self.name
 
 
+@python_2_unicode_compatible
+class InlineTestParentModel(models.Model):
+
+    name = models.CharField(
+        max_length = 100,
+    )
+
+    generic_children = GenericRelation(InlineTestChildGenericModel)
+
+    def __str__(self):
+        return self.name
+
+
+@python_2_unicode_compatible
 class InlineTestChildModel(models.Model):
 
     parent = models.ForeignKey(
@@ -124,4 +163,4 @@ class InlineTestUnrelatedParentModel(models.Model):
 
 class InlineTestUnrelatedChildModel(models.Model):
 
-    pass
+    parent = models.ForeignKey(InlineTestUnrelatedParentModel)
